{*************************************************************}
{ Unit for packing/unpacking of memory under the indexes on a }
{ segment (up to 64 Kilobytes) and packing/unpacking of files }
{ (any size). Compressing/decompressing functions - TASM 2.5  }
{ --------------                                              }
{ Compression method - modified RLE.                          }
{*************************************************************}
{ Version:   2.6                                              }
{ Author:    Aleksey Kuznetsov, Kiev, Ukraine                 }
{ E-Mail:    info@utilmind.com                                }
{ Homepage:  http://www.utilmind.com                          }
{ Created:   July, xx, 1994                                   }
{ Modified:  February, 2, 1998                                }
{ Legal:     Copyright (c) 1994-99, UtilMind Solutions        }
{*************************************************************}

{$I-}
{$L SFPACK}
unit SFPack;

interface

function SFastPack(SourceSeg, TargetSeg, SourceSize: Word): Word;
function SFastUnPack(SourceSeg, TargetSeg, SourceSize: Word): Word;
function PackFile(Source, Target: String): Boolean;
function UnPackFile(Source, Target: String): Boolean;

implementation

var
  FSize: LongInt;
  Tmp: Word;
  SourceBuf, TargetBuf: Pointer;
  SourceFile, TargetFile: File;

function SFastPack(SourceSeg, TargetSeg, SourceSize: Word): Word; external;

function SFastUnPack(SourceSeg, TargetSeg, SourceSize: Word): Word; external;

function PackFile(Source, Target: String): Boolean;
begin
  GetMem(SourceBuf, $FFFA);
  GetMem(TargetBuf, $FFFF);
  Assign(SourceFile, Source);
  Reset(SourceFile, 1);
  FSize := FileSize(SourceFile);
  Assign(TargetFile, Target);
  Rewrite(TargetFile, 1);
  while FSize <> 0 do
   begin
    if FSize > $FFFA then Tmp:=$FFFA
    else Tmp := FSize;
    dec(FSize, Tmp);
    BlockRead(SourceFile, SourceBuf^, Tmp);
    Tmp := SFastPack(Seg(SourceBuf^), Seg(TargetBuf^),Tmp);
    BlockWrite(TargetFile, Tmp, 2);
    BlockWrite(TargetFile, TargetBuf^, Tmp);
   end;
  Close(TargetFile);
  Close(SourceFile);
  FreeMem(TargetBuf, $FFFF);
  FreeMem(SourceBuf, $FFFA);
  if IOResult=0 then PackFile := True else PackFile := False;
end;

function UnPackFile(Source, Target: String): Boolean;
begin
  GetMem(SourceBuf, $FFFF);
  GetMem(TargetBuf, $FFFA);
  Assign(SourceFile, Source);
  Reset(SourceFile, 1);
  FSize:=FileSize(SourceFile);
  Assign(TargetFile, Target);
  Rewrite(TargetFile, 1);
  while FSize <> 0 do
   begin
    BlockRead(SourceFile, Tmp, 2);
    BlockRead(SourceFile, SourceBuf^, Tmp);
    dec(FSize, Tmp + 2);
    Tmp := SFastUnPack(Seg(SourceBuf^), Seg(TargetBuf^), Tmp);
    BlockWrite(TargetFile, TargetBuf^, Tmp);
   end;
  Close(TargetFile);
  Close(SourceFile);
  FreeMem(TargetBuf, $FFFA);
  FreeMem(SourceBuf, $FFFF);
  if IOResult = 0 then UnPackFile := True else UnPackFile := False;
end;

End.